"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useHasHydrated = exports.useIsHydrating = exports.useMarkAsHydrated = exports.useBoundaryHydrater = exports.HydrationProvider = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const retil_boundary_1 = require("retil-boundary");
const retil_mount_1 = require("retil-mount");
const retil_source_1 = require("retil-source");
const hydrationEnvService_1 = require("./hydrationEnvService");
const OnHydrationDetectedContext = react_1.createContext(null);
const HydratingContext = react_1.createContext(undefined);
/**
 * An optional provider that allows for manual control of the hydration state.
 *
 * When not provided, the hydration hooks will instead create and interact with
 * a default global hydration service.
 */
const HydrationProvider = (props) => {
    const { children, onHydrationDetected, isHydrating } = props;
    return (react_1.default.createElement(HydratingContext.Provider, { value: isHydrating },
        react_1.default.createElement(OnHydrationDetectedContext.Provider, { value: onHydrationDetected }, children)));
};
exports.HydrationProvider = HydrationProvider;
/**
 * Sets up retil-boundary based hydration detection for your app.
 *
 * This should be called once at the top level of your app, above any
 * <Boundary> components. It'll then caue `useIsHydrating()` to become `false`
 * after the first render of the deepest <Boundary> component.
 */
function useBoundaryHydrater() {
    const markAsHydrated = useMarkAsHydrated();
    retil_boundary_1.useBoundaryEffect(markAsHydrated);
}
exports.useBoundaryHydrater = useBoundaryHydrater;
/**
 * Returns a function that can be called to manually mark the app as hydrated.
 *
 * When used in conjunction with a `<HydrationProvider>`, this will call the
 * provided `onHydrationDetected` function. Otherwise, it'll call the default
 * hydration service's `hydrate` method.
 */
function useMarkAsHydrated() {
    const contextOnHydrationDetected = react_1.useContext(OnHydrationDetectedContext);
    return contextOnHydrationDetected || hydrationEnvService_1.getDefaultHydrationEnvService()[1];
}
exports.useMarkAsHydrated = useMarkAsHydrated;
/**
 * Pulls the latest hydration state from the first available source out of:
 *
 * - React Context (as set by a <HydrationProvider>)
 * - retil-mount's useEnv hook, if hydration state is set on the env
 * - the default hydration env service
 *
 *
 */
function useIsHydrating() {
    var _a;
    const env = retil_mount_1.useEnv();
    const contextHydrating = react_1.useContext(HydratingContext);
    const source = hydrationEnvService_1.getDefaultHydrationEnvService()[0];
    const serviceHydrating = retil_source_1.useSource(env ? null : source, {
        defaultValue: retil_mount_1.createEnvVector([env]),
    });
    return contextHydrating !== null && contextHydrating !== void 0 ? contextHydrating : (_a = serviceHydrating === null || serviceHydrating === void 0 ? void 0 : serviceHydrating[1]) === null || _a === void 0 ? void 0 : _a.hydrating;
}
exports.useIsHydrating = useIsHydrating;
function useHasHydrated() {
    return useIsHydrating() === false;
}
exports.useHasHydrated = useHasHydrated;
